--- src/Exmap.hpp.orig	Thu Sep 28 17:52:25 2006
+++ src/Exmap.hpp	Mon Jan 14 15:24:21 2008
@@ -13,6 +13,10 @@
 #include <boost/smart_ptr.hpp>
 
 #include <sys/types.h>
+#ifdef __FreeBSD__
+#include <kvm.h>
+#endif
+
 #include "jutil.hpp"
 #include "Elf.hpp"
 
@@ -73,7 +77,32 @@
     private:
     };
 
-    /// Concrete implementation of Sysinfo for a Linux system.
+#ifdef __FreeBSD__
+    /// Concrete implementation of Sysinfo for a FreeBSD system.
+    class FbsdSysInfo : public SysInfo
+    {
+    public:
+	FbsdSysInfo();
+	virtual ~FbsdSysInfo();
+	virtual std::list<pid_t> accessible_pids();
+	virtual bool sanity_check();
+	virtual bool read_page_info(pid_t pid,
+			    std::map<Elf::Address, std::list<Page> > &pi);
+	virtual std::string read_cmdline(pid_t pid);
+	virtual bool read_vmas(const PagePoolPtr &pp,
+			       pid_t pid,
+			       std::list<VmaPtr> &vmas);
+   protected:
+	PageCookie get_page_cookie();
+	static const int FbsdSysInfo::MAX_MAPS;
+   private:
+	PageCookie _page_cookie;
+	int	_modid;
+	int	_pmap_helper_syscall_num;
+	kvm_t*	_kd;
+	char	_kvm_errbuf[_POSIX2_LINE_MAX];
+   };
+#else/// Concrete implementation of Sysinfo for a Linux system.
     class LinuxSysInfo : public SysInfo
     {
     public:
@@ -96,11 +125,10 @@
 		bool &writable,
 		PageCookie &cookie);
     private:
-
         std::string proc_map_file(pid_t pid);
 	static const std::string EXMAP_FILE;
     };
-
+#endif
 
     /// Holds the various measures we can make of a File, Process or
     /// ELF memory range. Sizes are measured as doubles, to avoid too much
